unit MMIOFile;

interface

uses
  Windows,
  SysUtils,
  MMSystem;

const
  MMIO_INVALIDHANDLE = 0;

type
  mmAccess = (mmaRead, mmaWrite, mmaReadWrite);
  mmShare = (mmsCompat, mmsExclusive, mmsDenyWrite, mmsDenyRead, mmsDenyNone);
  TMMIOFile = class
    constructor Create;
    destructor Free;
    procedure Assign(FileName: String);
    procedure SetFileMode(Buffered: Boolean; AccessMode: mmAccess; ShareMode: mmShare);
    procedure SetBufferSize(Size: LongInt);
    procedure Reset;
    procedure Rewrite;
    function SeekTo(Position: LongInt): LongInt;
		function SeekBy(Distance: LongInt): LongInt;
		function SeekBOF: LongInt;
		function SeekEOF: LongInt;
    function Read(Buffer: Pointer; Count: Integer): Integer;
    function Write(Buffer: Pointer; Count: Integer): Integer;
    procedure Flush(ClearBuf: Boolean);
    procedure Rename(OldName, NewName: String);
    function Erase: Boolean;
    procedure Close;
    function Exists: Boolean;
    function FileName: String;
    function FilePos: LongInt;
  private
    mmioHandle: HMMIO;
    mmioInfo: PMMIOInfo;
    mmioAccessMode: mmAccess;
    mmioShareMode: mmShare;
    mmioBuffered, isOpen: Boolean;
    mmioOpenFlags: Cardinal;
    PName: PChar;
    procedure SetOpenFlags;
    procedure CheckHandle;
    procedure GetInfo;
    procedure SetInfo;
  end;
  EMMIOError = class(Exception)
  end;

implementation

constructor TMMIOFile.Create;
begin
  isOpen := false;
  mmioInfo := NIL;
  mmioHandle := MMIO_INVALIDHANDLE;
end;

destructor TMMIOFile.Free;
begin
  if isOpen then Close
end;

procedure TMMIOFile.Assign(FileName: String);
begin
  if not Assigned(mmioInfo)
  then New(mmioInfo);
  FillChar(mmioInfo^, SizeOf(mmioInfo^), 0);
  PName := StrNew(PChar(FileName));
  mmioAccessMode := mmaReadWrite;
  mmioShareMode := mmsExclusive;
  mmioBuffered := true;
end;

procedure TMMIOFile.SetFileMode(Buffered: Boolean; AccessMode: mmAccess; ShareMode: mmShare);
begin
  mmioAccessMode := AccessMode;
	mmioShareMode := ShareMode;
  mmioBuffered := Buffered;
end;

procedure TMMIOFile.SetBufferSize(Size: LongInt);
begin
  case mmioSetBuffer(mmioHandle, NIL, Size, 0) of
  MMIOERR_CANNOTWRITE:
    raise EMMIOError.Create('Could not write buffer to disk.');
  MMIOERR_OUTOFMEMORY:
    raise EMMIOError.Create('Could not allocate new buffer size.');
  end
end;

procedure TMMIOFile.SetOpenFlags;
begin
  if mmioBuffered
  then mmioOpenFlags := mmioOpenFlags or MMIO_ALLOCBUF;
  case mmioAccessMode of
  mmaRead:
    mmioOpenFlags := mmioOpenFlags or MMIO_READ;
  mmaWrite:
    mmioOpenFlags := mmioOpenFlags or MMIO_WRITE;
  mmaReadWrite:
    mmioOpenFlags := mmioOpenFlags or MMIO_READWRITE;
  end;
  case mmioShareMode of
	mmsCompat:
    mmioOpenFlags := mmioOpenFlags or MMIO_COMPAT;
  mmsExclusive:
    mmioOpenFlags := mmioOpenFlags or MMIO_EXCLUSIVE;
  mmsDenyWrite:
    mmioOpenFlags := mmioOpenFlags or MMIO_DENYWRITE;
  mmsDenyRead:
    mmioOpenFlags := mmioOpenFlags or MMIO_DENYREAD;
  mmsDenyNone:
    mmioOpenFlags := mmioOpenFlags or MMIO_DENYNONE;
  end;
end;

procedure TMMIOFile.CheckHandle;
begin
  if mmioHandle = MMIO_INVALIDHANDLE
  then begin
    case mmioInfo^.wErrorRet of
    MMIOERR_ACCESSDENIED:
      raise EMMIOError.Create('Access denied - this file is protected.');
    MMIOERR_INVALIDFILE:
      raise EMMIOError.Create('Invalid file - could not be opened.');
    MMIOERR_NETWORKERROR:
      raise EMMIOError.Create('Network error - request failed.');
    MMIOERR_PATHNOTFOUND:
      raise EMMIOError.Create('Path not found - invalid location.');
    MMIOERR_SHARINGVIOLATION:
			raise EMMIOError.Create('Sharing violation - file is in use.');
    MMIOERR_TOOMANYOPENFILES:
      raise EMMIOError.Create('Too many open files - no more handles.');
    end
  end
  else isOpen := true
end;

procedure TMMIOFile.Reset;
begin
  mmioOpenFlags := 0;
  SetOpenFlags;
  mmioHandle := mmioOpen(PName, mmioInfo, mmioOpenFlags);
  CheckHandle
end;

procedure TMMIOFile.Rewrite;
begin
  mmioOpenFlags := MMIO_CREATE;
  SetOpenFlags;
  mmioHandle := mmioOpen(PName, mmioInfo, mmioOpenFlags);
  CheckHandle
end;

function TMMIOFile.SeekTo(Position: LongInt): LongInt;
begin
  Result := mmioSeek(mmioHandle, Position, SEEK_SET)
end;

function TMMIOFile.SeekBy(Distance: LongInt): LongInt;
begin
	Result := mmioSeek(mmioHandle, Distance, SEEK_CUR)
end;

function TMMIOFile.SeekBOF: LongInt;
begin
	Result := mmioSeek(mmioHandle, 0, SEEK_SET)
end;

function TMMIOFile.SeekEOF: LongInt;
begin
	Result := mmioSeek(mmioHandle, 0, SEEK_END)
end;

function TMMIOFile.Read(Buffer: Pointer; Count: Integer): Integer;
begin
  Result := mmioRead(mmioHandle, Buffer, Count)
end;

function TMMIOFile.Write(Buffer: Pointer; Count: Integer): Integer;
begin
  Result := mmioWrite(mmioHandle, Buffer, Count)
end;

procedure TMMIOFile.Flush(ClearBuf: Boolean);
begin
  case mmioFlush(mmioHandle, Ord(ClearBuf)*MMIO_EMPTYBUF) of
  MMIOERR_CANNOTWRITE:
    raise EMMIOError.Create('Could not write buffer to disk.');
  end;
end;

procedure TMMIOFile.Rename(OldName, NewName: String);
begin
  mmioRename(PChar(OldName), PChar(NewName), NIL, 0)
end;

function TMMIOFile.Erase: Boolean;
begin
  if Assigned(PName)
  then Result := Boolean(mmioOpen(PName, mmioInfo, MMIO_DELETE))
  else Result := false
end;

procedure TMMIOFile.Close;
begin
  case mmioClose(mmioHandle, 0) of
  MMIOERR_CANNOTWRITE:
    raise EMMIOError.Create('Could not write buffer to disk.');
  end
end;

procedure TMMIOFile.GetInfo;
begin
  mmioGetInfo(mmioHandle, mmioInfo, 0)
end;

procedure TMMIOFile.SetInfo;
begin
  mmioSetInfo(mmioHandle, mmioInfo, 0)
end;

function TMMIOFile.Exists: Boolean;
begin
  if Assigned(PName)
  then Result := Boolean(mmioOpen(PName, mmioInfo, MMIO_EXIST))
  else Result := false
end;

function TMMIOFile.FileName: String;
begin
  Result := String(PName)
end;

function TMMIOFile.FilePos: LongInt;
begin
  Result := SeekBy(0)
end;

end.

