unit WavEngU;

interface

uses
  Windows,
  MMIOFile,
  Math,
  WaveObj;

function GetEngineVersion(var VerNr: Word): PChar;
export; stdcall;

function ResetFilePointer(mmFile: TMMIOFile): LongWord;
export; stdcall;
// Reset file pointer to the begin of file

function MoveFilePointer(mmFile: TMMIOFile; Dist: LongInt): LongWord;
export; stdcall;
// Moves the file pointer from the current position

function GetFilePointer(mmFile: TMMIOFile): LongWord;
export; stdcall;
// Returns the current file pointer position

function OpenSampleFile(FileName: PChar): TMMIOFile;
export; stdcall;
// opens an existing sample file

function CreateSampleFile(FileName: PChar; AlwaysOverwrite: Boolean): TMMIOFile;
export; stdcall;
// creates a new sample file

function ReadWaveHeader(mmFile: TMMIOFile; var WH: TWaveHeader; var Samples: LongWord): LongWord;
export; stdcall;
// reads RIFF header, sample count and required WAVE chunks from WAVE file
// into a TWaveHeader structure and an unsigned long integer;
// returns data offset in WAVE file or 0

function WriteWaveHeader(mmFile: TMMIOFile; var WH: TWaveHeader; Samples: LongWord): LongWord;
export; stdcall;
// writes RIFF header and required WAVE chunks from WAVE file
// from a TWaveHeader structure; calculates expected sizes;
// returns data offset for WAVE file or 0

function ReadIntBlock(mmFile: TMMIOFile; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;
// reads a block of up to (ICount) integer sample values from WAVE file
// into a buffer for long integer values;
// returns the real count of read values

function ReadFloatBlock(mmFile: TMMIOFile; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;
// reads a block of up to (FCount) float sample values from WAVE file;
// returns the real count of read values

procedure Int2FloatBlock(IBuf: PInt; FBuf: PFloat; Count: LongWord; ValidBits: Byte);
export; stdcall;

procedure Float2IntBlock(FBuf: PFloat; IBuf: PInt; Count: LongWord; ValidBits: Byte);
export; stdcall;

function WriteIntBlock(mmFile: TMMIOFile; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;

function WriteFloatBlock(mmFile: TMMIOFile; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;

function Sign(Value : Single): ShortInt;
export; stdcall;

function dB(r: Single): Single;
export; stdcall;

function dB1(r: Single): Single;
export; stdcall;

function TanH(r: Single): Single;
export; stdcall;

function ArTanH(r: Single): Single;
export; stdcall;

procedure GetValueRange(FBuf: PFloat; Count: LongWord; var Min, Max: Single);
export; stdcall;

procedure ProcessSampleBlock(FBuf: PFloat; Count: LongWord; var Callback: TPerSampleFunction);
export; stdcall;

implementation

uses
  SysUtils, Controls, Dialogs;

function GetEngineVersion(var VerNr: Word): PChar;
export; stdcall;
begin
  VerNr := $0052;
  Result := 'WavEngine DLL - Version 0.52 beta - 2002/07/29 by *LigH*'
end;

function ResetFilePointer(mmFile: TMMIOFile): LongWord;
export; stdcall;
// Reset file pointer to the begin of file
begin
	Result := mmFile.SeekBOF
end;

function MoveFilePointer(mmFile: TMMIOFile; Dist: LongInt): LongWord;
export; stdcall;
// Moves the file pointer from the current position
begin
  Result := mmFile.SeekBy(Dist)
end;

function GetFilePointer(mmFile: TMMIOFile): LongWord;
// Returns the current file pointer position
export; stdcall;
begin
  Result := mmFile.SeekBy(0)
end;

function OpenSampleFile(FileName: PChar): TMMIOFile;
export; stdcall;
// opens an existing sample file
var
  mmFile: TMMIOFile;
begin
  mmFile := TMMIOFile.Create;
  mmFile.Assign(String(FileName));
  mmFile.SetFileMode(true, mmaRead, mmsDenyWrite);
  try
    mmFile.Reset
  except
    on E:EMMIOError do begin
      MessageDlg(E.Message, mtError, [mbOK], 0);
      FreeAndNil(mmFile)
    end
  end;
  Result := mmFile
end;

function CreateSampleFile(FileName: PChar; AlwaysOverwrite: Boolean): TMMIOFile;
export; stdcall;
// creates a new sample file
var
  mmFile: TMMIOFile;
  OvS: String;
begin
  OvS := 'File "'+String(FileName)+'" exists, overwrite?';
  mmFile := TMMIOFile.Create;
  mmFile.Assign(String(FileName));
  mmFile.SetFileMode(true, mmaWrite, mmsDenyWrite);
  if AlwaysOverwrite
  or (mmFile.Exists and (MessageDlg(OvS, mtConfirmation, [mbYes, mbNo], 0) = mrYes))
  then try
    mmFile.Rewrite
  except
    on E:EMMIOError do begin
      MessageDlg(E.Message, mtError, [mbOK], 0);
      FreeAndNil(mmFile)
    end
  end;
  Result := mmFile
end;

function ReadWaveHeader(mmFile: TMMIOFile; var WH: TWaveHeader; var Samples: LongWord): LongWord;
export; stdcall;
// reads RIFF header, sample count and required WAVE chunks from WAVE file
// into a TWaveHeader structure and an unsigned long integer;
// returns data offset in WAVE file or 0
var
  RBytes: LongWord;
  Chunk: TChunk;
begin
  Result := 0;
  Samples := 0;
  ResetFilePointer(mmFile);
  RBytes := mmFile.Read(@WH.RIFF, SizeOf(TRIFFHeader));
  if (RBytes = SizeOf(TRIFFHeader))
  and (WH.RIFF.RIFF_ID.Value = rmRIFF.Value)
  and (WH.RIFF.SubType.Value = rtWave.Value)
  then while mmFile.Read(@Chunk, SizeOf(TChunk)) = SizeOf(TChunk) do begin
    if Chunk.ChunkID.Value = chFmt.Value
    then begin
      Move(Chunk, WH.fmt, SizeOf(TChunk));
      if WH.fmt.Size < 16
      then WH.fmt.wBitsPerSample := WH.fmt.wBlockAlign*8 div WH.fmt.wChannels;
      if WH.fmt.Size > 16
      then WH.fmt.Size := 16;
      if mmFile.Read(@WH.fmt.wFormatTag, WH.fmt.Size) < WH.fmt.Size
      then Exit;
      if Chunk.Size > WH.fmt.Size
      then begin
        if MoveFilePointer(mmFile, Chunk.Size-WH.fmt.Size) = SEEK_ERROR
        then Exit
      end
    end
    else if Chunk.ChunkID.Value = chData.Value
    then begin
      Move(Chunk, WH.data, SizeOf(TChunk));
      if (WH.fmt.ChunkID.Value = chFmt.Value)
      and (WH.fmt.wFormatTag in [WAVE_FORMAT_PCM, WAVE_FORMAT_IEEE_FLOAT])
      then Samples := WH.data.Size div (WH.fmt.wBlockAlign*WH.fmt.wChannels);
      Result := GetFilePointer(mmFile);
      Exit
    end
    else begin
      if MoveFilePointer(mmFile, Chunk.Size) = SEEK_ERROR
      then Exit
    end
  end
end;

function WriteWaveHeader(mmFile: TMMIOFile; var WH: TWaveHeader; Samples: LongWord): LongWord;
export; stdcall;
// writes RIFF header and required WAVE chunks from WAVE file
// from a TWaveHeader structure; calculates expected sizes;
// returns data offset for WAVE file or 0
var
  WBytes, Error: LongWord;
  Msg: PChar;
begin
  Result := 0;
  WH.RIFF.RIFF_ID := rmRIFF;
  WH.RIFF.SubType := rtWAVE;
  WH.fmt.ChunkID := chFmt;
  WH.fmt.Size := SizeOf(WH.fmt)-8;
  WH.data.ChunkID := chData;
  WH.data.Size := Samples*WH.fmt.wBlockAlign*WH.fmt.wChannels;
  WH.RIFF.Size := SizeOf(WH.RIFF.SubType)
                + SizeOf(WH.fmt.ChunkID) +SizeOf(WH.fmt.Size) +WH.fmt.Size
                + SizeOf(WH.data.ChunkID)+SizeOf(WH.data.Size)+WH.data.Size;
  ResetFilePointer(mmFile);
  if (mmFile.Write(@WH.RIFF, SizeOf(WH.RIFF)) = SizeOf(WH.RIFF))
  and (mmFile.Write(@WH.fmt, SizeOf(TChunk)) = SizeOf(TChunk))
  and (mmFile.Write(@WH.fmt.wFormatTag, WH.fmt.Size) = WH.fmt.Size)
  and (mmFile.Write(@WH.data, SizeOf(TChunk)) = SizeOf(TChunk))
  then Result := GetFilePointer(mmFile)
  else begin
    Error := GetLastError;
    Msg := nil;
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ALLOCATE_BUFFER,
                  nil, Error, 0, Msg, 0, nil);
    WriteLn('*ERROR*: ', Msg)
  end
end;

function ReadIntBlock(mmFile: TMMIOFile; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;
// reads a block of up to (ICount) integer sample values from WAVE file
// into a buffer for long integer values;
// returns the real count of read values
var
  ISample: LongInt;
  IBufPtr: PInt;
  ISmpPtr: PByte;
  ISmpIdx, RBytes: LongWord;
begin
  ISmpPtr := Addr(ISample);
  if WH.fmt.wBitsPerSample <= 24 then Inc(ISmpPtr);
  if WH.fmt.wBitsPerSample <= 16 then Inc(ISmpPtr);
  IBufPtr := IBuf;
  ISample := 0;
  ISmpIdx := 0;
  while (ISmpIdx < ICount)
  and (mmFile.Read(ISmpPtr, WH.fmt.wBlockAlign) = WH.fmt.wBlockAlign)
  do begin
    IBufPtr^ := ISample;
    Inc(IBufPtr);
    Inc(ISmpIdx)
  end;
  Result := ISmpIdx
end;

function ReadFloatBlock(mmFile: TMMIOFile; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;
// reads a block of up to (FCount) float sample values from WAVE file;
// returns the real count of read values
var
  RBytes: LongWord;
begin
  RBytes := mmFile.Read(FBuf, FCount*SizeOf(Single));
  Result := RBytes div SizeOf(Single)
end;

procedure Int2FloatBlock(IBuf: PInt; FBuf: PFloat; Count: LongWord; ValidBits: Byte);
export; stdcall;
var
  IBufPtr: PInt;
  FBufPtr: PFloat;
  BufIdx: LongWord;
begin
  IBufPtr := IBuf;
  FBufPtr := FBuf;
  for BufIdx := 0 to Pred(Count) do begin
    FBufPtr^ := IBufPtr^/Div32b;
    Inc(IBufPtr);
    Inc(FBufPtr)
  end
end;

procedure Float2IntBlock(FBuf: PFloat; IBuf: PInt; Count: LongWord; ValidBits: Byte);
export; stdcall;
var
  FBufPtr: PFloat;
  IBufPtr: PInt;
  BufIdx: LongWord;
begin
  FBufPtr := FBuf;
  IBufPtr := IBuf;
  for BufIdx := 0 to Pred(Count) do begin
    IBufPtr^ := Trunc(FBufPtr^*Div32b);
    Inc(IBufPtr);
    Inc(FBufPtr)
  end
end;

function WriteIntBlock(mmFile: TMMIOFile; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;
var
  ISample: LongInt;
  IBufPtr: PInt;
  ISmpPtr: PByte;
  ISmpIdx, WBytes: LongWord;
  ValidMask, VShiftIdx: Byte;
  written: Boolean;
begin
  ISmpPtr := Addr(ISample);
  if WH.fmt.wBitsPerSample <= 24 then Inc(ISmpPtr);
  if WH.fmt.wBitsPerSample <= 16 then Inc(ISmpPtr);
  ValidMask := $FF;
  {$IFOPT R+}{$DEFINE RC}{$ENDIF}
  {$R-}
  for VShiftIdx := 1 to ((32-WH.fmt.wBitsPerSample) mod 8)
  do ValidMask := (ValidMask shl 1);
  {$IFDEF RC}{$R+}{$UNDEF RC}{$ENDIF}
  IBufPtr := IBuf;
  ISample := 0;
  ISmpIdx := 0;
  repeat
    ISample := IBufPtr^;
    ISmpPtr^ := ISmpPtr^ and ValidMask;
    Inc(IBufPtr);
    written := (mmFile.Write(ISmpPtr, WH.fmt.wBlockAlign) = WH.fmt.wBlockAlign);
    if written then Inc(ISmpIdx)
  until (not written)
  or (ISmpIdx >= ICount)
  or (WBytes = 0);
  Result := ISmpIdx
end;

function WriteFloatBlock(mmFile: TMMIOFile; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;
var
  WBytes: LongWord;
begin
  WBytes := mmFile.Write(FBuf, FCount*SizeOf(Single));
  Result := WBytes div SizeOf(Single)
end;

function Sign(Value : Single): ShortInt;
export; stdcall;
begin
  if Value = 0.0 then Result := 0
  else if Value > 0.0 then Result := 1
  else Result := -1
end;

function dB(r: Single): Single;
export; stdcall;
begin
  if r = 0
  then Result := -256
  else Result := 20*Log10(Abs(r))
end;

function dB1(r: Single): Single;
export; stdcall;
begin
  Result := -Power(10, r/20)
end;

function TanH(r: Single): Single;
export; stdcall;
begin
  Result := 1.0-2.0/(1.0+Exp(2.0*r))
end;

function ArTanH(r: Single): Single;
export; stdcall;
begin
  Result := 0.5*ln((1+r)/(1-r))
end;

procedure GetValueRange(FBuf: PFloat; Count: LongWord; var Min, Max: Single);
export; stdcall;
var
  FBufPtr: PFloat;
  FBufIdx: LongWord;
begin
  Min := 1;
  Max := -1;
  FBufPtr := FBuf;
  for FBufIdx := 1 to Count do begin
    if FBufPtr^ < Min then Min := FBufPtr^;
    if FBufPtr^ > Max then Max := FBufPtr^;
    Inc(FBufPtr)
  end
end;

procedure ProcessSampleBlock(FBuf: PFloat; Count: LongWord; var Callback: TPerSampleFunction);
export; stdcall;
var
  FBufPtr: PFloat;
  FBufIdx: LongWord;
begin
  FBufPtr := FBuf;
  for FBufIdx := 1 to Count do begin
    FBufPtr^ := Callback(FBufPtr^);
    Inc(FBufPtr)
  end
end;

begin
end.

